@@ -13,7 +13,6 @@ import Control.Monad (guard, liftM2)
 
 import Crypto.PubKey.HashDescr (HashDescr(..))
 import qualified Crypto.PubKey.DSA as DSA
-import qualified Crypto.PubKey.RSA as RSA
 import qualified Crypto.PubKey.RSA.PKCS15 as P15
 
 import Data.ByteString (ByteString)
@@ -118,7 +117,7 @@ verify kr sig mt payload = do
 	verify''' f pub = if f then Right pub else Left "verification failed"
 	dsaVerify mpis ha pkey bs = DSA.verify (dsaTruncate pkey . hashFunction (hashDescr ha)) pkey (dsaMPIsToSig mpis) bs
 	rsaVerify mpis ha pkey bs = P15.verify (hashDescr ha) pkey bs (rsaMPItoSig mpis)
-        dsaMPIsToSig mpis = (unMPI (mpis !! 0), unMPI (mpis !! 1))
+        dsaMPIsToSig mpis = (DSA.Signature (unMPI (mpis !! 0)) (unMPI (mpis !! 1)))
         rsaMPItoSig mpis = integerToBEBS (unMPI (head mpis))
         finalPayload s pl = B.concat [pl, sigbit s, trailer s]
         sigbit s = runPut $ putPartialSigforSigning s
@@ -129,7 +128,7 @@ verify kr sig mt payload = do
         trailer s@(SignaturePkt (SigV4 {})) = runPut $ putSigTrailer s
         trailer _ = B.empty
         dsaTruncate pkey bs = if countBits bs > dsaQLen pkey then B.take (fromIntegral (dsaQLen pkey) `div` 8) bs else bs -- FIXME: uneven bits
-        dsaQLen pk = (\(_,_,z) -> countBits (integerToBEBS z)) (DSA.public_params pk)
+        dsaQLen = countBits . integerToBEBS . DSA.params_q . DSA.public_params
 	pkaAndMPIs (SigV4 _ pka _ _ _ _ mpis) = (pka,mpis)
 	pkaAndMPIs _ = error "This should never happen."
         isSignatureExpired :: Pkt -> Maybe UTCTime -> Either String Bool
